#include <stdlib.h>
#include <GL/glut.h>

GLfloat fAspect;
GLfloat cameraX = 40;
GLfloat cameraY = 60;
GLfloat cameraZ = 100;

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor  
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

	// Altera a cor do desenho para preto
	glColor3f(0.0f, 0.0f, 1.0f);
      
        // Funo da GLUT para fazer o desenho de um cubo 
	// com a cor corrente
	glutWireCube(50);
        
	glFlush();
}

// Funo usada para especificar o volume de visualizao
void EspecificaParametrosVisualizacao(void)
{
	// Especifica sistema de coordenadas de projeo
	glMatrixMode(GL_PROJECTION);
	// Inicializa sistema de coordenadas de projeo
	glLoadIdentity();

	// Especifica a projeo perspectiva(angulo,aspecto,zMin,zMax)
	gluPerspective(60,1.0,0.5,500);

	// Especifica sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	// Inicializa sistema de coordenadas do modelo
	glLoadIdentity();

	// Especifica posio do observador e do alvo
	gluLookAt(cameraX, cameraY, cameraZ, 0,0,0, 0,1,0);
}

// Funo callback chamada quando o tamanho da janela  alterado 
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	// Para previnir uma diviso por zero
	if ( h == 0 ) h = 1;

	// Especifica as dimenses da viewport
	glViewport(0, 0, w, h);
 
	// Calcula a correo de aspecto
	fAspect = (GLfloat)w/(GLfloat)h;

	EspecificaParametrosVisualizacao();
}

// Funo callback chamada para gerenciar eventos de teclas
void Teclado (unsigned char key, int x, int y)
{
	if (key == 27)
		exit(0);
    if (key == 'A') {
    	cameraX += 10;
    	EspecificaParametrosVisualizacao();
    	Desenha();
    }
    if (key == 'D') {
    	cameraX -= 10;
    	EspecificaParametrosVisualizacao();
    	Desenha();
    }
        
}

// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{   
	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glLineWidth(2.0);
}

// Programa Principal 
int main(int argc, char *argv[])
{

        glutInit(&argc, argv);	

        // Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB); 

	// Especifica a posio inicial da janela GLUT
	glutInitWindowPosition(5,5); 

	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(450,450); 
 
	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Desenho de um cubo");

	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);

	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);

	// Registra a funo callback para tratamento das teclas ASCII
	glutKeyboardFunc (Teclado);

	// Chama a funo responsvel por fazer as inicializaes 
	Inicializa();

 	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();

	return 0;
}
