#include <GL/glut.h>
#include <stdlib.h>

// Variveis para controles de navegao
GLfloat angle, fAspect;
GLfloat rotX, rotY, rotX_ini, rotY_ini;
GLfloat obsX, obsY, obsZ, obsX_ini, obsY_ini, obsZ_ini;
int x_ini,y_ini,bot;

// Define um vrtice
typedef struct {
	float x,y,z;	// posio no espao
} VERT;

// Define uma face
typedef struct {
	int total;	// total de vrtices
	int ind[4];	// ndices para o vetor de vrtices
} FACE;

// Define um objeto 3D
typedef struct {
	VERT *vertices;		// aponta para os vrtices
	FACE *faces;		// aponta para as faces
	int total_faces;	// total de faces no objeto
} OBJ;

// Definio dos vrtices
VERT vertices[] = {
	{ -1, 0, -1 },	// 0 canto inf esquerdo tras.
	{  1, 0, -1 },	// 1 canfo inf direito  tras.
	{  1, 0,  1 },	// 2 canto inf direito  diant.
	{ -1, 0,  1 },  // 3 canto inf esquerdo diant.
	{  0, 2,  0 },  // 4 topo
};

// Definio das faces
FACE faces[] = {
	{ 4, { 0,1,2,3 }},	// base
	{ 3, { 0,1,4,-1 }},	// lado traseiro
	{ 3, { 0,3,4,-1 }},	// lado esquerdo
	{ 3, { 1,2,4,-1 }},	// lado direito
	{ 3, { 3,2,4,-1 }}	// lado dianteiro
};

// Finalmente, define o objeto pirmide
OBJ piramide = {
	vertices, faces, 5 };

// Desenha um objeto em wireframe
void DesenhaObjetoWireframe(OBJ *objeto)
{
	OBJ *obj = objeto;
	// Percorre todas as faces
	int f;
	glColor3f(1.0f, 0.0f, 0.0f);
	glBegin(GL_LINE_STRIP);
	for(f=0; f < obj->total_faces; ++f)
	{	
		// Percorre todos os vrtices da face
		int v;
		for(v=0; v < obj->faces[f].total; ++v) {
			glVertex3f(obj->vertices[faces[f].ind[v]].x,
				obj->vertices[faces[f].ind[v]].y,
				obj->vertices[faces[f].ind[v]].z);
        }
		
	}
	glEnd();
}

// Funo callback de redesenho da janela de visualizao
void Desenha(void)
{
	// Limpa a janela de visualizao com a cor  
	// de fundo definida previamente
	glClear(GL_COLOR_BUFFER_BIT);

	// Altera a cor do desenho para preto
	glColor3f(0.0f, 0.0f, 0.0f);

	// Desenha o objeto definido anteriormente: uma pirmide
	DesenhaObjetoWireframe(&piramide);

	// Executa os comandos OpenGL
	glFlush();
}

// Funo usada para especificar a posio do observador virtual
void PosicionaObservador(void)
{
	// Especifica sistema de coordenadas do modelo
	glMatrixMode(GL_MODELVIEW);
	// Inicializa sistema de coordenadas do modelo
	glLoadIdentity();
	// Posiciona e orienta o observador
	glTranslatef(-obsX,-obsY,-obsZ);
	glRotatef(rotX,1,0,0);
	glRotatef(rotY,0,1,0);
}

// Funo usada para especificar o volume de visualizao
void EspecificaParametrosVisualizacao(void)
{
	// Especifica sistema de coordenadas de projeo
	glMatrixMode(GL_PROJECTION);
	// Inicializa sistema de coordenadas de projeo
	glLoadIdentity();

	// Especifica a projeo perspectiva(angulo,aspecto,zMin,zMax)
	gluPerspective(angle,fAspect,0.1,1200);

	PosicionaObservador();
}

// Funo callback chamada quando o tamanho da janela  alterado 
void AlteraTamanhoJanela(GLsizei w, GLsizei h)
{
	// Para previnir uma diviso por zero
	if ( h == 0 ) h = 1;

	// Especifica as dimenses da viewport
	glViewport(0, 0, w, h);
 
	// Calcula a correo de aspecto
	fAspect = (GLfloat)w/(GLfloat)h;

	EspecificaParametrosVisualizacao();
}

// Funo callback chamada para gerenciar eventos de teclas normais (ESC)
void Teclado (unsigned char tecla, int x, int y)
{
	if(tecla==27) // ESC ?
	{
		// Libera memria e finaliza programa
		exit(0);
	}
}

// Funo callback para tratar eventos de teclas especiais
void TeclasEspeciais (int tecla, int x, int y)  
{
	switch (tecla)
	{
		case GLUT_KEY_HOME:	if(angle>=10)  angle -=5;
							break;
		case GLUT_KEY_END:	if(angle<=150) angle +=5;
							break;
	}
	EspecificaParametrosVisualizacao();
	glutPostRedisplay();
}

// Funo callback para eventos de botes do mouse
void GerenciaMouse(int button, int state, int x, int y)
{
	if(state==GLUT_DOWN)
	{
		// Salva os parmetros atuais
		x_ini = x;
		y_ini = y;
		obsX_ini = obsX;
		obsY_ini = obsY;
		obsZ_ini = obsZ;
		rotX_ini = rotX;
		rotY_ini = rotY;
		bot = button;
	}
	else bot = -1;
}

// Funo callback para eventos de movimento do mouse
#define SENS_ROT	5.0
#define SENS_OBS	15.0
#define SENS_TRANSL	30.0
void GerenciaMovim(int x, int y)
{
	// Boto esquerdo ?
	if(bot==GLUT_LEFT_BUTTON)
	{
		// Calcula diferenas
		int deltax = x_ini - x;
		int deltay = y_ini - y;
		// E modifica ngulos
		rotY = rotY_ini - deltax/SENS_ROT;
		rotX = rotX_ini - deltay/SENS_ROT;
	}
	// Boto direito ?
	else if(bot==GLUT_RIGHT_BUTTON)
	{
		// Calcula diferena
		int deltaz = y_ini - y;
		// E modifica distncia do observador
		obsZ = obsZ_ini + deltaz/SENS_OBS;
	}
	// Boto do meio ?
	else if(bot==GLUT_MIDDLE_BUTTON)
	{
		// Calcula diferenas
		int deltax = x_ini - x;
		int deltay = y_ini - y;
		// E modifica posies
		obsX = obsX_ini + deltax/SENS_TRANSL;
		obsY = obsY_ini - deltay/SENS_TRANSL;
	}
	PosicionaObservador();
	glutPostRedisplay();
}

// Funo responsvel por inicializar parmetros e variveis
void Inicializa (void)
{	
	// Define a cor de fundo da janela de visualizao como branca
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
 
	// Inicializa a varivel que especifica o ngulo da projeo
	// perspectiva
	angle=60;
 
	// Inicializa as variveis usadas para alterar a posio do 
	// observador virtual
	rotX = 0;
	rotY = 0;
	obsX = obsY = 0;
	obsZ = 5;
}

// Programa Principal 
int main(int argc, char *argv[])
{
    glutInit(&argc, argv);
	// Define do modo de operao da GLUT
	glutInitDisplayMode(GLUT_SINGLE | GLUT_RGB); 
 
	// Especifica a posio inicial da janela GLUT
	glutInitWindowPosition(5,5); 
	 
	// Especifica o tamanho inicial em pixels da janela GLUT
	glutInitWindowSize(450,450); 
 
	// Cria a janela passando como argumento o ttulo da mesma
	glutCreateWindow("Desenho do wireframe de uma piramide");
 
	// Registra a funo callback de redesenho da janela de visualizao
	glutDisplayFunc(Desenha);
  
	// Registra a funo callback de redimensionamento da janela de visualizao
	glutReshapeFunc(AlteraTamanhoJanela);

	// Registra a funo callback para eventos de botes do mouse	
	glutMouseFunc(GerenciaMouse);
	 
	// Registra a funo callback para eventos de movimento do mouse	
	glutMotionFunc(GerenciaMovim);
	
	// Registra a funo callback para tratamento das teclas normais 
	glutKeyboardFunc (Teclado);

	// Registra a funo callback para tratamento das teclas especiais
	glutSpecialFunc (TeclasEspeciais);

	// Chama a funo responsvel por fazer as inicializaes 
	Inicializa();
 
	// Inicia o processamento e aguarda interaes do usurio
	glutMainLoop();
 
	return 0;
}
